﻿Imports System
Imports System.Net
Imports System.Net.Sockets
Imports System.Text
Imports System.IO
Imports System.Net.NetworkInformation

' [v2.2.4] New rtnStats format; 5 sec delay before writing report file to allow calling program to setup "Trigger on File Change" if used.
' [v2.2.5] Add arp phone entry delete to speed up the determination of state, otherwise delayed by 30-45 min arp cache auto refresh.
' [v2.2.6] Lengthened ping and wait times for better arp data
' [v2.2.7] Bug fix to recover from currupt findMyPhone.txt file
' [v2.2.8] Increase pings to 5; inc arp read delay to 30 sec
' [v2.3.3] As a last ditch effort, try a socket connection to see if phone is there.
' [v2.4.0] Added _newAlert flag to prevent overwhelming error message generation, and chgd -3 errRtn to a -4 for better handling.
' [v2.4.1] Fixed newAlert failure across separate executable runs with My.Settings
' [v2.4.2] Bug fix re lastKnows
' [v2.4.3] Reformat Returned info to elim extra line feeds.
' [v2.5.0] Bug fix on rtnStats, and elim name confusion by chg prior phone stat file read name into "lastKnownFile"
' [v2.5.1] Bug fix to correct missed phone from LAN.
' [v2.6.0] Added My.Settings.newAlert() for permanence across executions to prevent infinite error warning windows.
' [v3.0.0] New [beta] Version to support Smartphones rather than iPhone.
' [v3.1.0] Release of Smartphone Finder
' [v3.1.1] Reworked getIP to try and fix repeated err bug.
' [v3.1.2] Eliminated My.Settings and MsgBox actions which were jamming OS, if unanswered.
' [v3.1.3]Added delay after deleating arp file, brfore attempting to create new one, to assure file I/O completed.


Public Class findMyDroid

    Public _adptrName As String
    Public _rtnCode(3) As String
    Public _dvcMac As String
    Public _dvcIP As String
    Public _retryAddr As String
    Public rtnStats() As String = {"0", "Null data set", "", ""}
    Dim lines() As String = Nothing
    Dim _lastKnownIP As String = Nothing
    Dim _lastKnownMAC As String = Nothing
    Dim lastKnownFile() As String = Nothing



    Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load

        ' This routine probes the local network LAN to see if a Phone is connected to it.
        ' This is intended for use with Smartphones not including iPhones, which require special handling. 
        ' Smart phones can simply be pinged to discover, but the iPhone will not respond to a ping if asleep.
        ' The iPhone must be discovered indirectly by examning the Windows arp cache to see if there is a iPhone entry.
        ' 
        ' This routine is called (for example) like:
        '    "findMyPhone.exe 88-53-95-86-A0-1F C:/findMyPhone.txt" ,or
        '    "findMyPhone.exe 192.168.1.92 C:/findMyPhone.txt"
        ' Where:
        '  1st argument is the MAC or the IP address of the phone.
        '    
        '  2nd argument is the full path to a file where the Return Code is placed as a return from various Subs.
        '    1=success
        '   -1=No phone found in arp cache.
        '   -2=IP/MAC for dvc do not match arp cache
        '   -4=First detection of this phone with no accuracy check possible, or bad date in findMyPhone.txt file.  Make sure it is correct!
        '   -5=Winsock error
        '
        ' Use a IP Address if the phone has a STATIC IP address or the network router has A DHCP Reservation made to give the phone a fixed address.
        ' If the router has no phone DHCP Reservation (and the phone has its IP Address set by DHCP and not STATIC means) then use the MAC address,
        ' which will find the phone even if it's address changes every time it connects.
        '
        ' This routine will normally return a result within less than 5 seconds if the phone is in the arp cache, otherwise less than 45 seconds.
        ' A wait of 60 seconds should be plenty before attempting to read the findMyPhone.txt file to determine the results.
        '
        ' In the event of repeated -4 failures, to prevent overwhelming error messages from being generated, a variable "My.Settings.newAlert" is saved
        '  in the Registry between pgm executions to indicate if a warning has already been given. If 'newAlert' is FALSE no warning has been posted.
        '  When it is TRUE, a warning has been posted and thus no more will be, until the phone is discovered again, when newAlert will be reset to FALSE.

        ' Command$() returns any arguments posted in the program calling sequence.
        Try
            Dim bytes(1024) As Byte
            Dim commands As String = Command$()
            Dim parms() As String
            Dim temp(3) As String

            '            My.Settings.Reload()

            ' Initialize return info file data for 1st time usage.
            parms = commands.Split(New [Char]() {","c, " "c}, StringSplitOptions.RemoveEmptyEntries)
            Dim Addr As String = parms(0)
            Dim path As String = parms(1)
            'Dim macAddr As String = Command$()
            'Command$ nicely Trims the input to eliminate extra chars.

            If Addr = "" Then
                Exit Sub
            End If

            ' Get last known info on phone, if it exists.
            If File.Exists(path) Then
                lastKnownFile = File.ReadAllLines(path)
            Else
                ' There has to be a file there for subsequent checks, but all we can do for now is make it a nothing file.
                lastKnownFile = {"[RtnCode]", "0", "[Reason]", "Null data set for empty findMyPhone file", "[LastKnown]", "", ""}
                File.WriteAllLines(path, lastKnownFile)
            End If

            ' Split IP/MAC data out of last line in file.
            'temp = rtnStats(2).Split(New [Char]() {" "c}, StringSplitOptions.RemoveEmptyEntries)
            'Make sure that there are 9 elements and the last two are not empty.
            If lastKnownFile.Length = 7 Then
                _lastKnownIP = lastKnownFile(5)
                _lastKnownMAC = lastKnownFile(6)
                If (IsNothing(lastKnownFile(5)) Or IsNothing(lastKnownFile(6))) Then
                    ' The last entries are empty, so return with RtnCode of -4.
                    rtnStats(0) = "-4"
                End If
            Else
                ' Bad dataset, so return with RtnCode of -4.
                rtnStats(0) = "-4"
            End If

            Dim tries As Integer = 1
            While tries < 4
                ' If getIP() cannot obtain an MAC>IP translation it returns a -1.
                ' If the phone isn't initially found in the arp cache, it may have just timed out,
                ' so if no connect wait briefy and try one more time before returning with a failure code.

                If tries = 1 Then
                    getIP(Addr)
                ElseIf rtnStats(0) = "1" Then
                    ' We got an arp cache match, so done.
                    Exit While
                ElseIf tries = 2 And rtnStats(0) = "-1" Then
                    ' The getIP() routine did not find the sought phone meaning the phone is either AWAY, or
                    ' the phone has been inactive on the LAN for awhile and the arp cache timed out its entry.
                    '
                    ' To better determine if the arp cache entry has just timed out, or the phone is really AWAY,
                    ' we  will read the LastKnown entry and assume that is OK.  If not the next getIP() attemp will fail
                    ' with a RtnCode of -1. If that 2nd try fails then the phone is almost certainly AWAY.
                    '
                    ' Use lastKnown Addr info to attempt to locate the phone by pinging it, then checking the arp cache to see if the phone responded.
                    ' Even though an iPhone will not directly respond to a ping, the ping does cause the phone to refresh its arp cache entry.
                    Dim sendPing As New Ping
                    For k As Integer = 1 To 3
                        ' May need to ping multiple times if sleeping phone misses the 1st one.
                        Dim reply As PingReply = sendPing.Send(Addr)
                        ' Wait 500ms before next ping.
                        ' We don't need any response, we are just trying to poke the Phone to make it update the arp cache.
                        Threading.Thread.Sleep(500)
                    Next
                    ' Force 15 sec delay before checking the arp cache again.
                    Threading.Thread.Sleep(15000)
                    ' Now get arp info.
                    getIP(Addr)
                ElseIf tries = 3 And rtnStats(0) = "-1" Then
                    ' Two tries has not found the phone, thus the phone is almost certainly away from the LAN.
                    Exit While
                ElseIf rtnStats(0) = "-2" Then
                    ' IP address given does not match arp MAC. Use lastKnown MAC to try to find correct IP.
                    ' Call getIP() using MAC info to see if we can find the real phone.
                    getIP(_lastKnownMAC)
                    Exit While
                ElseIf rtnStats(0) = "-4" Then
                    ' This is a first use of this phone, or the lastKnown txt file is incomplete, so lastKnown data doesn't exist.
                    ' Accept the arp cache data but it is totally dependent on the given IP address being correct.
                    ' Give a success return but with caveat from getIP() in the Reasons detail.
                    rtnStats(0) = "1"
                    Exit While
                End If
                tries = tries + 1
            End While

Out:
            Dim strTries As String = tries - 1
            Dim rtnInfo() As String = {"[RtnCode]", rtnStats(0), "[Reason]", "After " + strTries + " try(s) " + rtnStats(1), "[LastKnown]", _lastKnownIP, _lastKnownMAC}
            ' Wait a second for the calling program to have time to get its act back together.
            Threading.Thread.Sleep(5000)
            File.WriteAllLines(path, rtnInfo)
            '
            ' Now delete phone entry from cache to speed up state chage detection.
            ' Otherwise it may be up to 45 min after phone is AWAY for the cache to auto detect it.
            ' This way it will have to be refreshed with each query.
            If rtnStats(0) = "1" Then
                ' Delete arp table if phone found to assure current (not cached) data on next try.
                Dim cmdArg As String = "-d " + _lastKnownIP
                If _lastKnownIP.Length > 0 Then RunCommandCom("arp", cmdArg, 0)
                ' Also reset newAlert error flag since we found the phone.
                '                My.Settings.newAlert = False
            End If
        Catch ex As Exception
            MsgBox("Error: " & ex.Message)
        End Try

        '        My.Settings.Save()
        Me.Close()
    End Sub


    Sub getIP(ByVal phoneAddr As String)

        ' If called with a MAC Address, then the phone's IP address is looked up in the Windows arp Table,
        ' using the MAC address as a search key.
        ' Return Codes
        '    1=success
        '   -1=No phone found in arp cache.
        '   -2=IP/MAC for dvc do not match arp cache
        '   -3=bad data in findMyPhone.txt file [replaced in v2.4.2 with a -4 return instead]
        '   -4=First detection of this phone with no accuracy check possible.  Make sure it is correct!
        '   -5=Winsock error


        Try
            Dim line, mncp, mnfp, incp, infp As String
            Dim _dvcIP, _dvcMAC As String
            Dim dvcInfo(3) As String
            Dim Address As String = UCase(phoneAddr)
            Dim addrNums(5)   'pure addr numbers found in arp
            Dim temp(25) As String
            Dim test As Integer
            Dim delArpEntry As String = Nothing
            Dim arpFile As String = "C:\arpCache.txt"

            ' Save calling address in case a retry is needed.
            'Get just the calling args input phoneAddr address numbers w/o separators (ie, colon/dash/space)
            Dim nums() As String = Address.Split(New [Char]() {".", ":"c, "-"c, " "c}, StringSplitOptions.RemoveEmptyEntries)
            'The file must be deleted or the new date will be added onto the end of prior contents.
            If File.Exists(arpFile) Then File.Delete(arpFile)
            Threading.Thread.Sleep(1000)  'give time for file I/O to complete before attempting to create new one.
            'Copy arp table output to file so we can examine it.
            RunCommandCom("arp -a >>", arpFile, 0)
            Threading.Thread.Sleep(3000)  'give a moment for response to return.
            Dim sr As New IO.StreamReader(arpFile)
            Dim s As String = sr.ReadToEnd
            Threading.Thread.Sleep(250)  'give a moment for response to return.
            sr.Close()
            lines = s.Split(New [Char]() {vbNewLine}, StringSplitOptions.RemoveEmptyEntries)

            If nums.Length = 4 Then   'Sub called with an IP address.
                ' Reform search IPAddress to a pure numbers only string for uncluttered match efort.
                incp = String.Concat(nums(0), nums(1), nums(2), nums(3))    'Pure IP digits
                For i As Integer = 2 To lines.Length - 1  'First few entries are headers
                    If i = lines.Length - 1 Then
                        rtnStats(0) = "-1"
                        rtnStats(1) = "No phone with IP address (" & Address & ") found in arp cache."
                        ' No  matching address found in arp cache, so exit sub and goose iPhone with pings
                        ' so we can retry once more after a short wait for the arp cache to repopulate.
                        Exit For
                    End If
                    ' line/temp() are values read from the arp cache.
                    line = UCase(lines(i))
                    temp = line.Split(New [Char]() {" "c}, StringSplitOptions.RemoveEmptyEntries)
                    _dvcIP = temp(1)
                    _dvcMAC = temp(2)
                    ' Convert arp segmented IP addr to a pure 8 digit number.
                    addrNums = _dvcIP.Split(New [Char]() {"."c, " "c}, StringSplitOptions.RemoveEmptyEntries)
                    infp = String.Concat(addrNums(0), addrNums(1), addrNums(2), addrNums(3))
                    ' infp from cache; incp from calling argument.
                    test = infp.IndexOf(incp)
                    If (test) <> -1 Then
                        ' A matching IP address was found in the arp cache
                        dvcInfo = line.Split(New [Char]() {" "c}, StringSplitOptions.RemoveEmptyEntries)
                        '
                        ' We matched the IP address, BUT if the LAN has DHCP enabled this address could now belong to a different device.
                        ' To be sure we have a correct IP/MAC match, use this found IP and make sure it matches the phone's MAC address.
                        ' A last known good IP/MAC pair was read from the findMyPhone.txt file. Confirm that data (_lastKnownMAC) matches the arp data just read (_dvcMAC).
                        If _lastKnownMAC <> dvcInfo(2) Then
                            rtnStats(0) = "-4"
                            rtnStats(1) = "No MAC Addr match.  May be first detection of this phone with no accuracy check possible or bad findMyPhone.txt file. Will now use this data, but make sure this MAC (" + dvcInfo(2) + ") is the same as your phone's."
                            _lastKnownIP = dvcInfo(1)
                            _lastKnownMAC = dvcInfo(2)
                            '                        If My.Settings.newAlert = False Then
                            '                            Dim dt As DateTime = DateTime.Now
                            '                            MsgBox(dt + ": Warning: Bad findMyPhone.txt file or a first detection of this phone with no accuracy check possible.  Will use this data, but make sure this MAC (" + dvcInfo(2) + ") is the same as your phone's.")
                            '                            My.Settings.newAlert = True
                            '                        End If

                        Else : _dvcMAC = _lastKnownMAC
                            ' Update rtnCode information for findMyPhone.txt file
                            rtnStats(0) = "1"
                            rtnStats(1) = "The phone was found on the network."
                            _lastKnownIP = dvcInfo(1)
                            _lastKnownMAC = dvcInfo(2)
                        End If
                        Exit For
                    End If
                Next
                Exit Sub
            ElseIf nums.Length = 6 Then  'We have a MAC address
                ' Reform macAddress to a pure numbers only string
                mncp = String.Concat(nums(0), nums(1), nums(2), nums(3), nums(4), nums(5))
                For i As Integer = 2 To lines.Length - 1  'First few entries are headers
                    If i = lines.Length - 1 Then
                        ' We got to end of cache list with no hits.
                        rtnStats(0) = "-1"
                        rtnStats(1) = "No phone with MAC (" & Address & ") found in arp cache."
                        ' No MAC matching address found in arp cache, so advise findMyPhone()
                        ' so it can retry once more after a short wait for the arp cache to repopulate.
                        Exit For
                    End If
                    line = UCase(lines(i))
                    temp = line.Split(New [Char]() {" "c}, StringSplitOptions.RemoveEmptyEntries)
                    _dvcIP = temp(1)
                    _dvcMAC = temp(2)
                    ' Convert segmented MAC addr to a pure 12 digit number.
                    addrNums = _dvcMAC.Split(New [Char]() {":"c, "-"c, " "c}, StringSplitOptions.RemoveEmptyEntries)
                    mnfp = String.Concat(addrNums(0), addrNums(1), addrNums(2), addrNums(3), addrNums(4), addrNums(5))
                    test = mnfp.IndexOf(mncp)
                    If (test) <> -1 Then
                        ' A matching MAC address was found in the arp cache
                        dvcInfo = line.Split(New [Char]() {" "c}, StringSplitOptions.RemoveEmptyEntries)
                        ' Update rtnCode information for findMyPhone.txt file
                        If rtnStats(0) = "-2" Then
                            rtnStats(1) = "Given IP addresss incorrect, but phone found on network by MAC! LastKnowns updated."
                        Else
                            _rtnCode(1) = "Reason: The phone was found on the network."
                        End If
                        rtnStats(0) = "1"
                        _lastKnownIP = dvcInfo(1)
                        _lastKnownMAC = dvcInfo(2)
                        Exit For
                    End If
                Next
            Else
                MsgBox("Phone network Address does not appear correct" & vbNewLine & vbNewLine & _
                       "It should have a form like " & """88-53-95-86-A0-1F"" or ""192.168.1.92""")
            End If

        Catch ex As Exception
            MsgBox("Error: " & ex.Message)
        End Try

    End Sub

    Sub socketCom(ByVal trgIP As String)
        ' Connect to a remote device.

        ' Establish the remote endpoint for the socket.
        ' The iPhone uses port 62078 for its control interactions.
        Try
            Dim ipAddr As IPAddress = IPAddress.Parse(trgIP)
            Dim remoteEP As New IPEndPoint(ipAddr, 62078)

            ' Create a TCP/IP socket.
            Dim mySocket As New Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp)
            'mySocket.SendTimeout() = 30000

            ' Connect the socket to the remote endpoint.
            mySocket.Connect(remoteEP)
            If Not mySocket.Connected Then
                rtnStats(0) = "-5"
                rtnStats(1) = "WinSock connection error"
                'MsgBox("Winsock error: " + Convert.ToString(System.Runtime.InteropServices.Marshal.GetLastWin32Error()))
            End If
            ' The socket effort succeeded, so return with a positive RtnCode
            rtnStats(0) = "1"

            ' Release the socket.
            mySocket.Shutdown(SocketShutdown.Both)
            mySocket.Close()

        Catch ex As Exception
            'MsgBox(ex.Message)
            ' The socket connect effort timed out without success, so the phone must really gone.
            rtnStats(0) = "-1"
            rtnStats(1) = "No phone with IP address (" & trgIP & ") found connected to LAN."
        End Try

    End Sub

    Sub RunCommandCom(ByVal command As String, ByVal arguments As String, ByVal permanent As Boolean)
        'This routine Runs executables as called from the command line.

        Try
            Dim p As Process = New Process()
            Dim pi As ProcessStartInfo = New ProcessStartInfo()
            pi.Arguments = " " & If(permanent = True, "/K", "/C") & " " & command & " " & arguments
            pi.FileName = "cmd.exe"
            pi.UseShellExecute = False
            p.StartInfo = pi
            p.Start()

        Catch ex As Exception
            MsgBox(ex.Message)
        End Try

    End Sub


    Sub getNetAdptr()
        'This routine is not used with findMyPhone but is included herre for future potential use.

        '
        ' getNetAdptr finds the Device Name of the active newtork adapter on this computer.
        ' It looks thru all the network adapters for the Adapter which is connected to the LAN Gateway
        ' (non "0.0.0.0" or null "::" Gateway address), and which has a status of "Up"
        '
        'The active Adapter name (eg, "Local Connection") is Returned in _adptrName

        Dim NetworkAdapters() As NetworkInterface = NetworkInterface.GetAllNetworkInterfaces
        Dim myAdapterProps As IPInterfaceProperties = Nothing
        Dim myGateways As GatewayIPAddressInformationCollection = Nothing

        For Each adapter As NetworkInterface In NetworkAdapters
            myAdapterProps = adapter.GetIPProperties
            myGateways = myAdapterProps.GatewayAddresses
            For Each Gateway As GatewayIPAddressInformation In myGateways
                If Not Gateway.Address.ToString = "0.0.0.0" And _
                    Not Gateway.Address.ToString = "::" And _
                    adapter.OperationalStatus.ToString() = "Up" Then
                    _adptrName = adapter.Name
                End If
            Next
        Next

        ' The above code would be used with the following arp manipulation efforts . . .
        '
        'If (dvcState = "DYNAMIC") Then
        '    Dim cmdStrDel As String = "delete neighbors """ & _adptrName & """ """ & dvcIP & """"
        '    Dim cmdStrAdd As String = "add neighbors """ & _adptrName & """ """ & dvcIP & """ """ & dvcMAC & """"
        '    Delete the DYNAMIC arp table setting for the iPhone and replace it with STATIC.
        '    A DYNAMIC entry must be Deleted from the arp cache before a STATIC can be added.
        '    To add a STATIC entry that resolves IP address 192.168.1.92 to the physical address 88-53-95-86-A0-1F'
        '     use the following syntax where:
        '    "Local Area Connection" is the actual adapter name serving this computer, plus the IP/MAC as appropriate.

        '        cmd netsh interface ipv4 delete neighbors "Local Area Connection" 192.168.1.92
        '        cmd netsh interface ipv4 add neighbors "Local Area Connection" "192.168.1.92" "88-53-95-86-A0-1F"

        '    RunCommandCom("netsh interface ipv4", cmdStrDel, 0)
        '    RunCommandCom("netsh interface ipv4", cmdStrAdd, 0)
        'End If

    End Sub



End Class
